@@ -42,6 +42,7 @@
 #include <libempathy/empathy-gsettings.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-dispatcher.h>
+#include <libempathy/empathy-marshal.h>
 
 #include "empathy-chat.h"
 #include "empathy-spell.h"
@@ -57,9 +58,10 @@
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
 
+
 #define CHAT_DIR_CREATE_MODE  (S_IRUSR | S_IWUSR | S_IXUSR)
 #define CHAT_FILE_CREATE_MODE (S_IRUSR | S_IWUSR)
-#define IS_ENTER(v) (v == GDK_Return || v == GDK_ISO_Enter || v == GDK_KP_Enter)
+#define IS_ENTER(v) (v == GDK_KEY_Return || v == GDK_KEY_ISO_Enter || v == GDK_KEY_KP_Enter)
 #define MAX_INPUT_HEIGHT 150
 #define COMPOSING_STOP_TIMEOUT 5
 
@@ -99,6 +101,9 @@ struct _EmpathyChatPriv {
 	gulong		   delete_range_id;
 	gulong		   notify_cursor_position_id;
 
+	/* Source func ID for update_misspelled_words () */
+	guint              update_misspelled_words_id;
+
 	GtkWidget         *widget;
 	GtkWidget         *hpaned;
 	GtkWidget         *vbox_left;
@@ -1105,7 +1110,9 @@ chat_state_changed_cb (EmpathyTpChat      *tp_chat,
 }
 
 static void
-chat_message_received (EmpathyChat *chat, EmpathyMessage *message)
+chat_message_received (EmpathyChat *chat,
+	EmpathyMessage *message,
+	gboolean pending)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	EmpathyContact  *sender;
@@ -1124,7 +1131,7 @@ chat_message_received (EmpathyChat *chat, EmpathyMessage *message)
 			       chat);
 
 	priv->unread_messages++;
-	g_signal_emit (chat, signals[NEW_MESSAGE], 0, message);
+	g_signal_emit (chat, signals[NEW_MESSAGE], 0, message, pending);
 }
 
 static void
@@ -1132,7 +1139,7 @@ chat_message_received_cb (EmpathyTpChat  *tp_chat,
 			  EmpathyMessage *message,
 			  EmpathyChat    *chat)
 {
-	chat_message_received (chat, message);
+	chat_message_received (chat, message, FALSE);
 }
 
 static void
@@ -1480,15 +1487,15 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 
 	/* Catch ctrl+up/down so we can traverse messages we sent */
 	if ((event->state & GDK_CONTROL_MASK) &&
-	    (event->keyval == GDK_Up ||
-	     event->keyval == GDK_Down)) {
+	    (event->keyval == GDK_KEY_Up ||
+	     event->keyval == GDK_KEY_Down)) {
 		GtkTextBuffer *buffer;
 		const gchar   *str;
 
 		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
 		chat_input_history_update (chat, buffer);
 
-		if (event->keyval == GDK_Up) {
+		if (event->keyval == GDK_KEY_Up) {
 			str = chat_input_history_get_next (chat);
 		} else {
 			str = chat_input_history_get_prev (chat);
@@ -1534,24 +1541,24 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 		return FALSE;
 	}
 	if (!(event->state & GDK_CONTROL_MASK) &&
-	    event->keyval == GDK_Page_Up) {
+	    event->keyval == GDK_KEY_Page_Up) {
 		adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (text_view_sw));
 		gtk_adjustment_set_value (adj, gtk_adjustment_get_value (adj) - gtk_adjustment_get_page_size (adj));
 		return TRUE;
 	}
 	if ((event->state & GDK_CONTROL_MASK) != GDK_CONTROL_MASK &&
-	    event->keyval == GDK_Page_Down) {
+	    event->keyval == GDK_KEY_Page_Down) {
 		adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (text_view_sw));
 		val = MIN (gtk_adjustment_get_value (adj) + gtk_adjustment_get_page_size (adj),
 			   gtk_adjustment_get_upper (adj) - gtk_adjustment_get_page_size (adj));
 		gtk_adjustment_set_value (adj, val);
 		return TRUE;
 	}
-	if (event->keyval == GDK_Escape) {
+	if (event->keyval == GDK_KEY_Escape) {
 		empathy_search_bar_hide (EMPATHY_SEARCH_BAR (priv->search_bar));
 	}
 	if (!(event->state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)) &&
-	    event->keyval == GDK_Tab) {
+	    event->keyval == GDK_KEY_Tab) {
 		GtkTextBuffer *buffer;
 		GtkTextIter    start, current;
 		gchar         *nick, *completed;
@@ -1906,6 +1913,7 @@ chat_log_filter (TplEntry *log,
 
 	for (; pending; pending = g_list_next (pending)) {
 		if (empathy_message_equal (message, pending->data)) {
+			g_object_unref (message);
 			return FALSE;
 		}
 	}
@@ -1932,7 +1940,7 @@ show_pending_messages (EmpathyChat *chat) {
 
 	for (l = messages; l != NULL ; l = g_list_next (l)) {
 		EmpathyMessage *message = EMPATHY_MESSAGE (l->data);
-		chat_message_received (chat, message);
+		chat_message_received (chat, message, TRUE);
 	}
 }
 
@@ -2291,6 +2299,7 @@ static gboolean
 update_misspelled_words (gpointer data)
 {
 	EmpathyChat *chat = EMPATHY_CHAT (data);
+	EmpathyChatPriv *priv = GET_PRIV (chat);
 	GtkTextBuffer *buffer;
 	GtkTextIter iter;
 	gint length;
@@ -2301,6 +2310,9 @@ update_misspelled_words (gpointer data)
 	length = gtk_text_iter_get_offset (&iter);
 	chat_input_text_buffer_insert_text_cb (buffer, &iter,
 					       NULL, length, chat);
+
+	priv->update_misspelled_words_id = 0;
+
 	return FALSE;
 }
 
@@ -2331,7 +2343,8 @@ conf_spell_checking_cb (GSettings *gsettings_chat,
 			/* Possibly changed dictionaries,
 			 * update misspelled words. Need to do so in idle
 			 * so the spell checker is updated. */
-			g_idle_add (update_misspelled_words, chat);
+			priv->update_misspelled_words_id =
+				g_idle_add (update_misspelled_words, chat);
 		}
 
 		return;
@@ -2364,7 +2377,8 @@ conf_spell_checking_cb (GSettings *gsettings_chat,
 
 		/* Mark misspelled words in the existing buffer.
 		 * Need to do so in idle so the spell checker is updated. */
-		g_idle_add (update_misspelled_words, chat);
+		priv->update_misspelled_words_id =
+			g_idle_add (update_misspelled_words, chat);
 	} else {
 		GtkTextTagTable *table;
 		GtkTextTag *tag;
@@ -2583,6 +2597,9 @@ chat_finalize (GObject *object)
 
 	DEBUG ("Finalized: %p", object);
 
+	if (priv->update_misspelled_words_id != 0)
+		g_source_remove (priv->update_misspelled_words_id);
+
 	g_object_unref (priv->gsettings_chat);
 	g_object_unref (priv->gsettings_ui);
 
@@ -2739,9 +2756,9 @@ empathy_chat_class_init (EmpathyChatClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__OBJECT,
+			      _empathy_marshal_VOID__OBJECT_BOOLEAN,
 			      G_TYPE_NONE,
-			      1, EMPATHY_TYPE_MESSAGE);
+			      2, EMPATHY_TYPE_MESSAGE, G_TYPE_BOOLEAN);
 
 	g_type_class_add_private (object_class, sizeof (EmpathyChatPriv));
 }
@@ -3232,6 +3249,11 @@ empathy_chat_paste (EmpathyChat *chat)
 
 	priv = GET_PRIV (chat);
 
+	if (gtk_widget_get_visible (priv->search_bar)) {
+		empathy_search_bar_paste_clipboard (EMPATHY_SEARCH_BAR (priv->search_bar));
+		return;
+	}
+
 	if (priv->tp_chat == NULL ||
 	    !gtk_widget_is_sensitive (chat->input_text_view))
 		return;
