@@ -178,7 +178,6 @@ empathy_contact_information_dialog_show (EmpathyContact *contact,
 
 	/* Create dialog */
 	dialog = gtk_dialog_new ();
-	empathy_dialog_remove_separator (GTK_DIALOG (dialog));
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_window_set_title (GTK_WINDOW (dialog),
 		empathy_contact_get_alias (contact));
@@ -238,7 +237,6 @@ empathy_contact_edit_dialog_show (EmpathyContact *contact,
 
 	/* Create dialog */
 	dialog = gtk_dialog_new ();
-	empathy_dialog_remove_separator (GTK_DIALOG (dialog));
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Edit Contact Information"));
 
@@ -290,7 +288,6 @@ empathy_contact_personal_dialog_show (GtkWindow *parent)
 
 	/* Create dialog */
 	personal_dialog = gtk_dialog_new ();
-	empathy_dialog_remove_separator (GTK_DIALOG (personal_dialog));
 	gtk_window_set_resizable (GTK_WINDOW (personal_dialog), FALSE);
 	gtk_window_set_title (GTK_WINDOW (personal_dialog), _("Personal Information"));
 
@@ -334,24 +331,28 @@ empathy_contact_personal_dialog_show (GtkWindow *parent)
  *  New contact dialog
  */
 
-static gboolean
-can_add_contact_to_account (TpAccount *account,
-			    gpointer   user_data)
+static void
+can_add_contact_to_account (TpAccount                                 *account,
+			    EmpathyAccountChooserFilterResultCallback  callback,
+			    gpointer                                   callback_data,
+			    gpointer                                   user_data)
 {
 	EmpathyContactManager *contact_manager;
 	TpConnection          *connection;
 	gboolean               result;
 
 	connection = tp_account_get_connection (account);
-	if (connection == NULL)
-		return FALSE;
+	if (connection == NULL) {
+		callback (FALSE, callback_data);
+		return;
+	}
 
 	contact_manager = empathy_contact_manager_dup_singleton ();
 	result = empathy_contact_manager_get_flags_for_connection (
 		contact_manager, connection) & EMPATHY_CONTACT_LIST_CAN_ADD;
 	g_object_unref (contact_manager);
 
-	return result;
+	callback (result, callback_data);
 }
 
 static void
@@ -396,7 +397,6 @@ empathy_new_contact_dialog_show_with_contact (GtkWindow *parent,
 
 	/* Create dialog */
 	dialog = gtk_dialog_new ();
-	empathy_dialog_remove_separator (GTK_DIALOG (dialog));
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_window_set_title (GTK_WINDOW (dialog), _("New Contact"));
 
