@@ -43,8 +43,10 @@ static GtkWidget *new_individual_dialog = NULL;
  *  New contact dialog
  */
 
-static gboolean
+static void
 can_add_contact_to_account (TpAccount *account,
+    EmpathyAccountChooserFilterResultCallback callback,
+    gpointer callback_data,
     gpointer user_data)
 {
   EmpathyIndividualManager *individual_manager;
@@ -53,14 +55,17 @@ can_add_contact_to_account (TpAccount *account,
 
   connection = tp_account_get_connection (account);
   if (connection == NULL)
-    return FALSE;
+    {
+      callback (FALSE, callback_data);
+      return;
+    }
 
   individual_manager = empathy_individual_manager_dup_singleton ();
   result = empathy_individual_manager_get_flags_for_connection (
     individual_manager, connection) & EMPATHY_INDIVIDUAL_MANAGER_CAN_ADD;
   g_object_unref (individual_manager);
 
-  return result;
+  callback (result, callback_data);
 }
 
 static void
@@ -107,7 +112,6 @@ empathy_new_individual_dialog_show_with_individual (GtkWindow *parent,
 
   /* Create dialog */
   dialog = gtk_dialog_new ();
-  empathy_dialog_remove_separator (GTK_DIALOG (dialog));
   gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
   gtk_window_set_title (GTK_WINDOW (dialog), _("New Contact"));
 
