@@ -180,7 +180,7 @@ individual_information_dialog_set_individual (
 
       /* Update the UI */
       gtk_window_set_title (GTK_WINDOW (dialog),
-          folks_individual_get_alias (individual));
+          folks_aliasable_get_alias (FOLKS_ALIASABLE (individual)));
       empathy_individual_widget_set_individual (
           EMPATHY_INDIVIDUAL_WIDGET (priv->individual_widget), individual);
       set_label_visibility (dialog);
@@ -273,7 +273,6 @@ empathy_individual_information_dialog_init (
   dialog->priv = priv;
   priv->individual = NULL;
 
-  empathy_dialog_remove_separator (GTK_DIALOG (dialog));
   gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
 
   content_area = GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog)));
