@@ -133,6 +133,8 @@ individual_menu_add_personas (GtkMenuShell *menu,
           folks_persona_get_display_id (FOLKS_PERSONA (persona)), account);
 
       contact_item = gtk_image_menu_item_new_with_label (label);
+      gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (contact_item),
+                                                 TRUE);
       contact_submenu = gtk_menu_new ();
       gtk_menu_item_set_submenu (GTK_MENU_ITEM (contact_item), contact_submenu);
       image = gtk_image_new_from_icon_name (
@@ -747,7 +749,7 @@ static void
 favourite_menu_item_toggled_cb (GtkCheckMenuItem *item,
   FolksIndividual *individual)
 {
-  folks_favourite_set_is_favourite (FOLKS_FAVOURITE (individual),
+  folks_favouritable_set_is_favourite (FOLKS_FAVOURITABLE (individual),
       gtk_check_menu_item_get_active (item));
 }
 
@@ -759,7 +761,7 @@ empathy_individual_favourite_menu_item_new (FolksIndividual *individual)
   item = gtk_check_menu_item_new_with_label (_("Favorite"));
 
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
-      folks_favourite_get_is_favourite (FOLKS_FAVOURITE (individual)));
+      folks_favouritable_get_is_favourite (FOLKS_FAVOURITABLE (individual)));
 
   g_signal_connect (item, "toggled",
       G_CALLBACK (favourite_menu_item_toggled_cb), individual);
