@@ -47,6 +47,7 @@
 #include "empathy-individual-view.h"
 #include "empathy-individual-menu.h"
 #include "empathy-individual-store.h"
+#include "empathy-contact-dialogs.h"
 #include "empathy-images.h"
 #include "empathy-linking-dialog.h"
 #include "empathy-cell-renderer-expander.h"
@@ -236,10 +237,10 @@ groups_change_group_cb (GObject *source,
     GAsyncResult *result,
     gpointer user_data)
 {
-  FolksGroups *groups = FOLKS_GROUPS (source);
+  FolksGroupable *groupable = FOLKS_GROUPABLE (source);
   GError *error = NULL;
 
-  folks_groups_change_group_finish (groups, result, &error);
+  folks_groupable_change_group_finish (groupable, result, &error);
   if (error != NULL)
     {
       g_warning ("failed to change group: %s", error->message);
@@ -370,14 +371,14 @@ real_drag_individual_received_cb (EmpathyIndividualView *self,
   if (!tp_strdiff (new_group, EMPATHY_INDIVIDUAL_STORE_FAVORITE))
     {
       /* Mark contact as favourite */
-      folks_favourite_set_is_favourite (FOLKS_FAVOURITE (individual), TRUE);
+      folks_favouritable_set_is_favourite (FOLKS_FAVOURITABLE (individual), TRUE);
       return;
     }
 
   if (!tp_strdiff (old_group, EMPATHY_INDIVIDUAL_STORE_FAVORITE))
     {
       /* Remove contact as favourite */
-      folks_favourite_set_is_favourite (FOLKS_FAVOURITE (individual), FALSE);
+      folks_favouritable_set_is_favourite (FOLKS_FAVOURITABLE (individual), FALSE);
 
       /* Don't try to remove it */
       old_group = NULL;
@@ -385,13 +386,13 @@ real_drag_individual_received_cb (EmpathyIndividualView *self,
 
   if (new_group != NULL)
     {
-      folks_groups_change_group (FOLKS_GROUPS (individual), new_group, TRUE,
+      folks_groupable_change_group (FOLKS_GROUPABLE (individual), new_group, TRUE,
           groups_change_group_cb, NULL);
     }
 
   if (old_group != NULL && action == GDK_ACTION_MOVE)
     {
-      folks_groups_change_group (FOLKS_GROUPS (individual), old_group,
+      folks_groupable_change_group (FOLKS_GROUPABLE (individual), old_group,
           FALSE, groups_change_group_cb, NULL);
     }
 }
@@ -624,7 +625,7 @@ individual_view_drag_motion (GtkWidget *widget,
         }
 
       if (individual != NULL &&
-          folks_individual_is_online (individual) &&
+          folks_presence_owner_is_online (FOLKS_PRESENCE_OWNER (individual)) &&
           (caps & EMPATHY_CAPABILITIES_FT))
         {
           gdk_drag_status (context, GDK_ACTION_COPY, time_);
@@ -891,6 +892,25 @@ individual_view_key_press_event_cb (EmpathyIndividualView *view,
       data->button = 0;
       data->time = event->time;
       g_idle_add (individual_view_popup_menu_idle_cb, data);
+    } else if (event->keyval == GDK_KEY_F2) {
+        FolksIndividual *individual;
+        EmpathyContact *contact;
+
+        g_return_val_if_fail (EMPATHY_IS_INDIVIDUAL_VIEW (view), FALSE);
+
+        individual = empathy_individual_view_dup_selected (view);
+        if (individual == NULL)
+            return FALSE;
+
+        contact = empathy_contact_dup_from_folks_individual (individual);
+        if (contact == NULL) {
+            g_object_unref (individual);
+            return FALSE;
+        }
+        empathy_contact_edit_dialog_show (contact, NULL);
+
+        g_object_unref (individual);
+        g_object_unref (contact);
     }
 
   return FALSE;
@@ -1314,7 +1334,8 @@ individual_view_search_key_navigation_cb (GtkWidget *search,
   GdkEventKey *eventkey = ((GdkEventKey *) event);
   gboolean ret = FALSE;
 
-  if (eventkey->keyval == GDK_Up || eventkey->keyval == GDK_Down)
+  if (eventkey->keyval == GDK_KEY_Up || eventkey->keyval == GDK_KEY_Down
+      || eventkey->keyval == GDK_KEY_F2)
     {
       GdkEvent *new_event;
 
@@ -1612,7 +1633,7 @@ individual_view_is_visible_individual (EmpathyIndividualView *self,
     return (priv->show_offline || is_online);
 
   /* check alias name */
-  str = folks_individual_get_alias (individual);
+  str = folks_aliasable_get_alias (FOLKS_ALIASABLE (individual));
 
   if (empathy_live_search_match (live, str))
     return TRUE;
@@ -2267,6 +2288,7 @@ empathy_individual_view_get_group_menu (EmpathyIndividualView *view)
   if (!group || is_fake_group)
     {
       /* We can't alter fake groups */
+      g_free (group);
       return NULL;
     }
 
@@ -2318,7 +2340,7 @@ individual_view_remove_activate_cb (GtkMenuItem *menuitem,
       text =
           g_strdup_printf (_
           ("Do you really want to remove the contact '%s'?"),
-          folks_individual_get_alias (individual));
+          folks_aliasable_get_alias (FOLKS_ALIASABLE (individual)));
       if (individual_view_remove_dialog_show (parent, _("Removing contact"),
               text))
         {
