@@ -20,6 +20,10 @@
  * Authors: Richard Hult <richard@imendio.com>
  *          Martyn Russell <martyn@imendio.com>
  *          Xavier Claessens <xclaesse@gmail.com>
+ *
+ * Some snippets are taken from GnuTLS 2.8.6, which is distributed under the
+ * same GNU Lesser General Public License 2.1 (or later) version. See
+ * empathy_get_x509_certified_hostname ().
  */
 
 #include "config.h"
@@ -374,8 +378,8 @@ create_errors_to_message_hash (void)
 		"cryptographically weak"));
 	g_hash_table_insert (errors, TP_ERROR_STR_CERT_LIMIT_EXCEEDED,
 		_("The length of the server certificate, or the depth of the "
-		"server certificate chain exceed the limits imposed by the "
-		"crypto library"));
+		"server certificate chain, exceed the limits imposed by the "
+		"cryptography library"));
 
 	return errors;
 }
@@ -396,13 +400,24 @@ empathy_dbus_error_name_get_default_message  (const gchar *error)
 }
 
 const gchar *
-empathy_account_get_error_message (TpAccount *account)
+empathy_account_get_error_message (TpAccount *account,
+    gboolean *user_requested)
 {
 	const gchar *dbus_error;
 	const gchar *message;
+        const GHashTable *details = NULL;
 	TpConnectionStatusReason reason;
 
-	dbus_error = tp_account_get_detailed_error (account, NULL);
+	dbus_error = tp_account_get_detailed_error (account, &details);
+
+        if (user_requested != NULL)
+          {
+            if (tp_asv_get_boolean (details, "user-requested", NULL))
+              *user_requested = TRUE;
+            else
+              *user_requested = FALSE;
+          }
+
 	message = empathy_dbus_error_name_get_default_message (dbus_error);
 	if (message != NULL)
 		return message;
@@ -723,8 +738,39 @@ empathy_contact_dup_from_folks_individual (FolksIndividual *individual)
 }
 
 TpChannelGroupChangeReason
-tp_chanel_group_change_reason_from_folks_groups_change_reason (
-    FolksGroupsChangeReason reason)
+tp_channel_group_change_reason_from_folks_groups_change_reason (
+    FolksGroupableChangeReason reason)
 {
   return (TpChannelGroupChangeReason) reason;
 }
+
+gchar *
+empathy_get_x509_certificate_hostname (gnutls_x509_crt_t cert)
+{
+  gchar dns_name[256];
+  gsize dns_name_size;
+  gint idx;
+  gint res = 0;
+
+  /* this snippet is taken from GnuTLS.
+   * see gnutls/lib/x509/rfc2818_hostname.c
+   */
+  for (idx = 0; res >= 0; idx++)
+    {
+      dns_name_size = sizeof (dns_name);
+      res = gnutls_x509_crt_get_subject_alt_name (cert, idx,
+          dns_name, &dns_name_size, NULL);
+
+      if (res == GNUTLS_SAN_DNSNAME || res == GNUTLS_SAN_IPADDRESS)
+        return g_strndup (dns_name, dns_name_size);
+    }
+
+  dns_name_size = sizeof (dns_name);
+  res = gnutls_x509_crt_get_dn_by_oid (cert, GNUTLS_OID_X520_COMMON_NAME,
+      0, 0, dns_name, &dns_name_size);
+
+  if (res >= 0)
+    return g_strndup (dns_name, dns_name_size);
+
+  return NULL;
+}
