@@ -26,12 +26,13 @@
 #include <string.h>
 
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <unique/unique.h>
 
-#if HAVE_LIBCHAMPLAIN
+#ifdef HAVE_LIBCHAMPLAIN
 #include <clutter-gtk/clutter-gtk.h>
 #endif
 
@@ -48,17 +49,14 @@
 
 #include <libempathy/empathy-idle.h>
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-account-settings.h>
 #include <libempathy/empathy-connectivity.h>
 #include <libempathy/empathy-connection-managers.h>
 #include <libempathy/empathy-dispatcher.h>
-#include <libempathy/empathy-dispatch-operation.h>
 #include <libempathy/empathy-ft-factory.h>
 #include <libempathy/empathy-gsettings.h>
 #include <libempathy/empathy-tp-chat.h>
-#include <libempathy/empathy-tp-call.h>
 
 #include <libempathy-gtk/empathy-ui-utils.h>
 #include <libempathy-gtk/empathy-location-manager.h>
@@ -68,8 +66,6 @@
 #include "empathy-accounts-dialog.h"
 #include "empathy-chat-manager.h"
 #include "empathy-status-icon.h"
-#include "empathy-call-window.h"
-#include "empathy-chat-window.h"
 #include "empathy-ft-manager.h"
 
 #include "extensions/extensions.h"
@@ -77,8 +73,6 @@
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include <libempathy/empathy-debug.h>
 
-#include <gst/gst.h>
-
 static gboolean start_hidden = FALSE;
 static gboolean no_connect = FALSE;
 
@@ -86,74 +80,6 @@ static void account_manager_ready_cb (GObject *source_object,
     GAsyncResult *result,
     gpointer user_data);
 
-static void
-dispatch_cb (EmpathyDispatcher *dispatcher,
-    EmpathyDispatchOperation *operation,
-    gpointer user_data)
-{
-  GQuark channel_type;
-
-  channel_type = empathy_dispatch_operation_get_channel_type_id (operation);
-
-  if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_TEXT)
-    {
-      EmpathyTpChat *tp_chat;
-      EmpathyChat   *chat = NULL;
-      const gchar   *id;
-
-      tp_chat = EMPATHY_TP_CHAT
-        (empathy_dispatch_operation_get_channel_wrapper (operation));
-
-      id = empathy_tp_chat_get_id (tp_chat);
-      if (!EMP_STR_EMPTY (id))
-        {
-          TpConnection *connection;
-          TpAccount *account;
-
-          connection = empathy_tp_chat_get_connection (tp_chat);
-          account = empathy_get_account_for_connection (connection);
-          chat = empathy_chat_window_find_chat (account, id);
-        }
-
-      if (chat)
-        {
-          empathy_chat_set_tp_chat (chat, tp_chat);
-        }
-      else
-        {
-          chat = empathy_chat_new (tp_chat);
-          /* empathy_chat_new returns a floating reference as EmpathyChat is
-           * a GtkWidget. This reference will be taken by a container
-           * (a GtkNotebook) when we'll call empathy_chat_window_present_chat */
-        }
-
-      empathy_chat_window_present_chat (chat,
-          empathy_dispatch_operation_get_user_action_time (operation));
-
-      if (empathy_tp_chat_is_invited (tp_chat, NULL))
-        {
-          /* We have been invited to the room. Add ourself as member as this
-           * channel has been approved. */
-          empathy_tp_chat_join (tp_chat);
-        }
-
-      empathy_dispatch_operation_claim (operation);
-    }
-  else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
-    {
-      EmpathyFTFactory *factory;
-
-      factory = empathy_ft_factory_dup_singleton ();
-
-      /* if the operation is not incoming, don't claim it,
-       * as it might have been triggered by another client, and
-       * we are observing it.
-       */
-      if (empathy_dispatch_operation_is_incoming (operation))
-        empathy_ft_factory_claim_channel (factory, operation);
-    }
-}
-
 static void
 use_conn_notify_cb (GSettings *gsettings,
     const gchar *key,
@@ -277,6 +203,11 @@ unique_app_message_cb (UniqueApp *unique_app,
   return UNIQUE_RESPONSE_OK;
 }
 
+static gboolean show_version_cb (const char *option_name,
+    const char *value,
+    gpointer data,
+    GError **error) G_GNUC_NORETURN;
+
 static gboolean
 show_version_cb (const char *option_name,
     const char *value,
@@ -286,8 +217,6 @@ show_version_cb (const char *option_name,
   g_print ("%s\n", PACKAGE_STRING);
 
   exit (EXIT_SUCCESS);
-
-  return FALSE;
 }
 
 static void
@@ -316,18 +245,6 @@ new_ft_handler_cb (EmpathyFTFactory *factory,
   g_object_unref (handler);
 }
 
-static void
-new_call_handler_cb (EmpathyCallFactory *factory,
-    EmpathyCallHandler *handler,
-    gboolean outgoing,
-    gpointer user_data)
-{
-  EmpathyCallWindow *window;
-
-  window = empathy_call_window_new (handler);
-  gtk_widget_show (GTK_WIDGET (window));
-}
-
 static void
 account_manager_ready_cb (GObject *source_object,
     GAsyncResult *result,
@@ -371,63 +288,6 @@ account_manager_ready_cb (GObject *source_object,
   g_object_unref (gsettings);
 }
 
-static EmpathyDispatcher *
-setup_dispatcher (void)
-{
-  EmpathyDispatcher *d;
-  GPtrArray *filters;
-  struct {
-    const gchar *channeltype;
-    TpHandleType handletype;
-  } types[] = {
-    /* Text channels with handle types none, contact and room */
-    { TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_NONE  },
-    { TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_CONTACT  },
-    { TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_ROOM  },
-    /* file transfer to contacts */
-    { TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER, TP_HANDLE_TYPE_CONTACT  },
-    /* roomlists */
-    { TP_IFACE_CHANNEL_TYPE_ROOM_LIST, TP_HANDLE_TYPE_NONE },
-  };
-  GHashTable *asv;
-  guint i;
-
-  /* Setup the basic Client.Handler that matches our client filter */
-  filters = g_ptr_array_new ();
-  asv = tp_asv_new (
-        TP_IFACE_CHANNEL ".ChannelType", G_TYPE_STRING,
-           TP_IFACE_CHANNEL_TYPE_TEXT,
-        TP_IFACE_CHANNEL ".TargetHandleType", G_TYPE_INT,
-            TP_HANDLE_TYPE_CONTACT,
-        NULL);
-  g_ptr_array_add (filters, asv);
-
-  d = empathy_dispatcher_new (PACKAGE_NAME, filters, NULL);
-
-  g_ptr_array_foreach (filters, (GFunc) g_hash_table_destroy, NULL);
-  g_ptr_array_free (filters, TRUE);
-
-  /* Setup the an extended Client.Handler that matches everything we can do */
-  filters = g_ptr_array_new ();
-  for (i = 0 ; i < G_N_ELEMENTS (types); i++)
-    {
-      asv = tp_asv_new (
-        TP_IFACE_CHANNEL ".ChannelType", G_TYPE_STRING, types[i].channeltype,
-        TP_IFACE_CHANNEL ".TargetHandleType", G_TYPE_INT, types[i].handletype,
-        NULL);
-
-      g_ptr_array_add (filters, asv);
-    }
-
-  empathy_dispatcher_add_handler (d, PACKAGE_NAME"MoreThanMeetsTheEye",
-    filters, NULL);
-
-  g_ptr_array_foreach (filters, (GFunc) g_hash_table_destroy, NULL);
-  g_ptr_array_free (filters, TRUE);
-
-  return d;
-}
-
 static void
 account_status_changed_cb (TpAccount *account,
     guint old_status,
@@ -437,15 +297,11 @@ account_status_changed_cb (TpAccount *account,
     GHashTable *details,
     EmpathyChatroom *room)
 {
-  TpConnection *conn;
-
-  conn = tp_account_get_connection (account);
-  if (conn == NULL)
+  if (new_status != TP_CONNECTION_STATUS_CONNECTED)
     return;
 
-  empathy_dispatcher_join_muc (conn,
-      empathy_chatroom_get_room (room), EMPATHY_DISPATCHER_NON_USER_ACTION,
-      NULL, NULL);
+  empathy_dispatcher_join_muc (account,
+      empathy_chatroom_get_room (room), TP_USER_ACTION_TIME_NOT_USER_ACTION);
 }
 
 static void
@@ -492,9 +348,9 @@ account_manager_chatroom_ready_cb (GObject *source_object,
             }
           else
             {
-              empathy_dispatcher_join_muc (conn,
+              empathy_dispatcher_join_muc (account,
                   empathy_chatroom_get_room (room),
-                  EMPATHY_DISPATCHER_NON_USER_ACTION, NULL, NULL);
+                  TP_USER_ACTION_TIME_NOT_USER_ACTION);
             }
         }
 
@@ -529,7 +385,7 @@ empathy_idle_set_auto_away_cb (GSettings *gsettings,
 int
 main (int argc, char *argv[])
 {
-#if HAVE_GEOCLUE
+#ifdef HAVE_GEOCLUE
   EmpathyLocationManager *location_manager = NULL;
 #endif
   EmpathyStatusIcon *icon;
@@ -537,7 +393,6 @@ main (int argc, char *argv[])
   TpAccountManager *account_manager;
   TplLogManager *log_manager;
   EmpathyChatroomManager *chatroom_manager;
-  EmpathyCallFactory *call_factory;
   EmpathyFTFactory  *ft_factory;
   GtkWidget *window;
   EmpathyIdle *idle;
@@ -570,14 +425,15 @@ main (int argc, char *argv[])
 
   /* Init */
   g_thread_init (NULL);
+
+#ifdef HAVE_LIBCHAMPLAIN
+  gtk_clutter_init (&argc, &argv);
+#endif
+
   empathy_init ();
 
   optcontext = g_option_context_new (N_("- Empathy IM Client"));
-  g_option_context_add_group (optcontext, gst_init_get_option_group ());
   g_option_context_add_group (optcontext, gtk_get_option_group (TRUE));
-#if HAVE_LIBCHAMPLAIN
-  g_option_context_add_group (optcontext, clutter_get_option_group ());
-#endif
   g_option_context_add_main_entries (optcontext, options, GETTEXT_PACKAGE);
 
   if (!g_option_context_parse (optcontext, &argc, &argv, &error)) {
@@ -591,7 +447,6 @@ main (int argc, char *argv[])
 
   empathy_gtk_init ();
   g_set_application_name (_(PACKAGE_NAME));
-  g_setenv ("PULSE_PROP_media.role", "phone", TRUE);
 
   gtk_window_set_default_icon_name ("empathy");
   textdomain (GETTEXT_PACKAGE);
@@ -606,10 +461,12 @@ main (int argc, char *argv[])
 
   if (unique_app_is_running (unique_app))
     {
-      unique_app_send_message (unique_app, UNIQUE_ACTIVATE, NULL);
-
-      g_object_unref (unique_app);
-      return EXIT_SUCCESS;
+      if (unique_app_send_message (unique_app, UNIQUE_ACTIVATE, NULL) ==
+          UNIQUE_RESPONSE_OK)
+        {
+          g_object_unref (unique_app);
+          return EXIT_SUCCESS;
+        }
     }
 
   notify_init (_(PACKAGE_NAME));
@@ -639,9 +496,9 @@ main (int argc, char *argv[])
   tp_account_manager_prepare_async (account_manager, NULL,
       account_manager_ready_cb, NULL);
 
-  /* Handle channels */
-  dispatcher = setup_dispatcher ();
-  g_signal_connect (dispatcher, "dispatch", G_CALLBACK (dispatch_cb), NULL);
+  /* The EmpathyDispatcher doesn't dispatch anything any more but we have to
+   * keep it around as we still use it to request channels */
+  dispatcher = empathy_dispatcher_dup_singleton ();
 
   migrate_config_to_xdg_dir ();
 
@@ -660,7 +517,6 @@ main (int argc, char *argv[])
   log_manager = tpl_log_manager_dup_singleton ();
 
   chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
-  empathy_chatroom_manager_observe (chatroom_manager, dispatcher);
 
   g_object_get (chatroom_manager, "ready", &chatroom_manager_ready, NULL);
   if (!chatroom_manager_ready)
@@ -673,10 +529,6 @@ main (int argc, char *argv[])
       chatroom_manager_ready_cb (chatroom_manager, NULL, account_manager);
     }
 
-  /* Create the call factory */
-  call_factory = empathy_call_factory_initialise ();
-  g_signal_connect (G_OBJECT (call_factory), "new-call-handler",
-      G_CALLBACK (new_call_handler_cb), NULL);
   /* Create the FT factory */
   ft_factory = empathy_ft_factory_dup_singleton ();
   g_signal_connect (ft_factory, "new-ft-handler",
@@ -684,8 +536,14 @@ main (int argc, char *argv[])
   g_signal_connect (ft_factory, "new-incoming-transfer",
       G_CALLBACK (new_incoming_transfer_cb), NULL);
 
+  if (!empathy_ft_factory_register (ft_factory, &error))
+    {
+      g_warning ("Failed to register FileTransfer handler: %s", error->message);
+      g_error_free (error);
+    }
+
   /* Location mananger */
-#if HAVE_GEOCLUE
+#ifdef HAVE_GEOCLUE
   location_manager = empathy_location_manager_dup_singleton ();
 #endif
 
@@ -705,12 +563,13 @@ main (int argc, char *argv[])
   g_object_unref (log_manager);
   g_object_unref (dispatcher);
   g_object_unref (chatroom_manager);
-#if HAVE_GEOCLUE
+#ifdef HAVE_GEOCLUE
   g_object_unref (location_manager);
 #endif
   g_object_unref (ft_factory);
   g_object_unref (unique_app);
   g_object_unref (gsettings);
+  gtk_widget_destroy (window);
 
   notify_uninit ();
   xmlCleanupParser ();
